=== modified file 'loggerhead/controllers/annotate_ui.py'
--- loggerhead/controllers/annotate_ui.py	2008-07-23 21:41:49 +0000
+++ loggerhead/controllers/annotate_ui.py	2008-08-16 21:53:52 +0000
@@ -62,6 +62,21 @@
             path = h.get_path(revid, file_id)
         filename = os.path.basename(path)
 
+        # Directory Breadcrumbs
+        directory_breadcrumbs = (
+            util.directory_breadcrumbs(
+                self._branch.friendly_name,
+                self._branch.is_root,
+                'files'))
+
+        # Create breadcrumb trail for the path within the branch
+        try:
+            inv = h.get_inventory(revid)
+        except:
+            self.log.exception('Exception fetching changes')
+            raise HTTPServerError('Could not fetch changes')
+        branch_breadcrumbs = util.branch_breadcrumbs(path, inv, 'files')
+
         return {
             'revid': revid,
             'file_id': file_id,
@@ -71,4 +86,6 @@
             'change': h.get_changes([ revid ])[0],
             'contents': list(h.annotate_file(file_id, revid)),
             'fileview_active': True,
+            'directory_breadcrumbs': directory_breadcrumbs,
+            'branch_breadcrumbs': branch_breadcrumbs,
         }

=== modified file 'loggerhead/controllers/changelog_ui.py'
--- loggerhead/controllers/changelog_ui.py	2008-07-22 00:40:03 +0000
+++ loggerhead/controllers/changelog_ui.py	2008-08-16 21:53:52 +0000
@@ -86,6 +86,13 @@
                     all_same_author = False
                     break
 
+        # Directory Breadcrumbs
+        directory_breadcrumbs = (
+            util.directory_breadcrumbs(
+                self._branch.friendly_name,
+                self._branch.is_root,
+                'changes'))
+
         return {
             'branch': self._branch,
             'changes': changes,
@@ -100,4 +107,5 @@
             'search_failed': search_failed,
             'all_same_author': all_same_author,
             'url': self._branch.context_url,
+            'directory_breadcrumbs': directory_breadcrumbs,
         }

=== modified file 'loggerhead/controllers/directory_ui.py'
--- loggerhead/controllers/directory_ui.py	2008-08-07 04:39:09 +0000
+++ loggerhead/controllers/directory_ui.py	2008-08-16 21:53:52 +0000
@@ -22,6 +22,7 @@
 
 from bzrlib import branch
 
+from loggerhead import util
 from loggerhead.controllers import TemplatedBranchView
 
 class DirEntry(object):
@@ -71,16 +72,13 @@
             dirs.append(DirEntry(d, parity, b))
             parity = 1 - parity
         # Create breadcrumb trail
-        dir_parts = self._name.strip('/').split('/')
-        breadcrumbs = []
-        for index, dir_name in enumerate(dir_parts):
-            breadcrumbs.append({
-                'dir_name': dir_name,
-                'path': '/'.join(dir_parts[:index + 1]),
-            })
+        directory_breadcrumbs = util.directory_breadcrumbs(
+                self._name,
+                False,
+                'directory')
         return {
             'dirs': dirs,
             'name': self._name,
             'static_url': static_url,
-            'breadcrumbs': breadcrumbs,
+            'directory_breadcrumbs': directory_breadcrumbs,
             }

=== modified file 'loggerhead/controllers/inventory_ui.py'
--- loggerhead/controllers/inventory_ui.py	2008-08-16 04:57:38 +0000
+++ loggerhead/controllers/inventory_ui.py	2008-08-16 21:53:52 +0000
@@ -76,14 +76,13 @@
             updir_file_id = None
 
         # Directory Breadcrumbs
-        (directory_breadcrumbs, root_name, root_suffix) = (
-            util.directory_breadcrumbs(
+        directory_breadcrumbs = util.directory_breadcrumbs(
                 self._branch.friendly_name,
                 self._branch.is_root,
-                'files'))
+                'files')
 
         # Create breadcrumb trail for the path within the branch
-        branch_breadcrumbs = util.branch_breadcrumbs(path, inv)
+        branch_breadcrumbs = util.branch_breadcrumbs(path, inv, 'files')
         
         return {
             'branch': self._branch,
@@ -103,6 +102,4 @@
             'fileview_active': True,
             'directory_breadcrumbs': directory_breadcrumbs,
             'branch_breadcrumbs': branch_breadcrumbs,
-            'root_name': root_name,
-            'root_suffix': root_suffix,
         }

=== modified file 'loggerhead/controllers/revision_ui.py'
--- loggerhead/controllers/revision_ui.py	2008-07-22 00:40:03 +0000
+++ loggerhead/controllers/revision_ui.py	2008-08-16 21:53:52 +0000
@@ -72,6 +72,13 @@
         if side_by_side:
             h.add_side_by_side([ change ])
 
+        # Directory Breadcrumbs
+        directory_breadcrumbs = (
+            util.directory_breadcrumbs(
+                self._branch.friendly_name,
+                self._branch.is_root,
+                'changes'))
+
         return {
             'branch': self._branch,
             'revid': revid,
@@ -89,4 +96,5 @@
             'line_count': line_count,
             'line_count_limit': line_count_limit,
             'show_plain_diffs': line_count > line_count_limit,
+            'directory_breadcrumbs': directory_breadcrumbs,
         }

=== modified file 'loggerhead/templates/annotate.pt'
--- loggerhead/templates/annotate.pt	2008-08-04 03:42:01 +0000
+++ loggerhead/templates/annotate.pt	2008-08-16 21:53:52 +0000
@@ -22,9 +22,9 @@
         </a>
       </tal:has-link>
       <tal:no-link condition="not: branch/branch_link">
-        <tal:branch-name replace="branch/friendly_name" />
+        <span metal:use-macro="breadcrumbs/directory"></span>
       </tal:no-link>
-      <span>: <span tal:content="path"></span> (revision <tal:revno content="change/revno"></tal:revno>)</span>
+      <span>: <span metal:use-macro="breadcrumbs/branch" /> (revision <tal:revno content="change/revno"></tal:revno>)</span>
     </h1>
 
     <div metal:fill-slot="content">

=== modified file 'loggerhead/templates/breadcrumbs.pt'
--- loggerhead/templates/breadcrumbs.pt	2008-08-16 04:57:38 +0000
+++ loggerhead/templates/breadcrumbs.pt	2008-08-16 21:53:52 +0000
@@ -9,13 +9,13 @@
 -->
 
 <!-- Branch breadcrumbs (for the path within the branch) -->
-<span metal:define-macro="branch">
-    <span tal:repeat="crumb branch_breadcrumbs" class="breadcrumb">/<a tal:attributes="href python:url(['/files', change.revno], file_id=crumb['file_id'])" tal:content="crumb/dir_name" /></span>
+<span metal:define-macro="branch" class="breadcrumb">
+    <tal:block repeat="crumb branch_breadcrumbs">/<a tal:attributes="href python:url([crumb['suffix'], change.revno], file_id=crumb['file_id'])" tal:content="crumb/dir_name" tal:condition="not:repeat/crumb/end" /><span tal:replace="crumb/dir_name" tal:condition="repeat/crumb/end" /></tal:block>
 </span>
 
 <!-- Directory breadcrumbs (for the path leading up to the branch) -->
-<span metal:define-macro="directory">
-    <span class="breadcrumb"><a tal:attributes="href string:/${root_suffix}" tal:content="root_name" /></span><span tal:repeat="crumb directory_breadcrumbs" class="breadcrumb">/<a tal:attributes="href string:/${crumb/path}${crumb/suffix}" tal:content="crumb/dir_name" /></span>
+<span metal:define-macro="directory" class="breadcrumb">
+    <tal:block repeat="crumb directory_breadcrumbs"><span tal:condition="not:repeat/crumb/start">/</span><a tal:attributes="href string:/${crumb/path}${crumb/suffix}" tal:content="crumb/dir_name" /></tal:block>
 </span>
 
 </body>

=== modified file 'loggerhead/templates/changelog.pt'
--- loggerhead/templates/changelog.pt	2008-08-04 03:42:01 +0000
+++ loggerhead/templates/changelog.pt	2008-08-16 21:53:52 +0000
@@ -21,7 +21,7 @@
              </a>
            </tal:has-link>
            <tal:no-link condition="not: branch/branch_link">
-             <tal:branch-name replace="branch/friendly_name" />
+             <span metal:use-macro="breadcrumbs/directory"></span>
            </tal:no-link>
            : changes
           <tal:block condition="filter_file_id">to <span tal:content="python:history.get_path(revid, filter_file_id)" /></tal:block>

=== modified file 'loggerhead/templates/directory.pt'
--- loggerhead/templates/directory.pt	2008-08-10 02:56:11 +0000
+++ loggerhead/templates/directory.pt	2008-08-16 21:53:52 +0000
@@ -9,8 +9,7 @@
     <div id="loggerheadCont">
       <h1>
         Browsing
-        <!-- This is all one one line to get the spacing right -->
-        <span class="breadcrumb"><a href="/">(root)</a></span><span tal:repeat="crumb breadcrumbs" class="breadcrumb">/<a tal:attributes="href string:/${crumb/path}" tal:content="crumb/dir_name" /></span>
+        <span metal:use-macro="breadcrumbs/directory" />
       </h1>
 
       <div >

=== modified file 'loggerhead/templates/revision.pt'
--- loggerhead/templates/revision.pt	2008-08-04 03:42:01 +0000
+++ loggerhead/templates/revision.pt	2008-08-16 21:53:52 +0000
@@ -23,7 +23,7 @@
             </a>
           </tal:has-link>
           <tal:no-link condition="not: branch/branch_link">
-            <tal:branch-name replace="branch/friendly_name" />
+            <span metal:use-macro="breadcrumbs/directory"></span>
           </tal:no-link>
           <span>: <tal:revno content="change/revno"></tal:revno>
         <tal:compare-to condition="python:compare_revid is not None">

=== modified file 'loggerhead/util.py'
--- loggerhead/util.py	2008-08-16 04:57:38 +0000
+++ loggerhead/util.py	2008-08-16 21:53:52 +0000
@@ -355,28 +355,35 @@
     """
     # Is our root directory itself a branch?
     if is_root:
-        breadcrumbs = []
-        root_name = path
-        root_suffix = 'files'
+        if view == 'directory':
+            directory = 'files'
+        breadcrumbs = [{
+            'dir_name': path,
+            'path': '',
+            'suffix': view,
+        }]
     else:
         # Create breadcrumb trail for the path leading up to the branch
-        breadcrumbs = []
-        dir_parts = path.strip('/').split('/')
-        for index, dir_name in enumerate(dir_parts):
-            breadcrumbs.append({
-                'dir_name': dir_name,
-                'path': '/'.join(dir_parts[:index + 1]),
-                'suffix': '',
-            })
-        # The branch link itself needs this or it will browse to the revision
-        # view instead of the file view
-        breadcrumbs[-1]['suffix'] = '/' + view
-        root_name = '(root)'
-        root_suffix = ''
-        
-    return (breadcrumbs, root_name, root_suffix)
+        breadcrumbs = [{
+            'dir_name': "(root)",
+            'path': '',
+            'suffix': '',
+        }]
+        if path != '/':
+            dir_parts = path.strip('/').split('/')
+            for index, dir_name in enumerate(dir_parts):
+                breadcrumbs.append({
+                    'dir_name': dir_name,
+                    'path': '/'.join(dir_parts[:index + 1]),
+                    'suffix': '',
+                })
+            # If we are not in the directory view, the last crumb is a branch,
+            # so we need to specify a view
+            if view != 'directory':
+                breadcrumbs[-1]['suffix'] = '/' + view
+    return breadcrumbs
 
-def branch_breadcrumbs(path, inv):
+def branch_breadcrumbs(path, inv, view):
     """
     Generate breadcrumb information from the branch path given
     
@@ -385,6 +392,7 @@
     Arguments:
     path -- The path to convert into breadcrumbs
     inv -- Inventory to get file information from
+    view -- The type of view we are showing (files, changes etc)
     """
     dir_parts = path.strip('/').split('/')
     inner_breadcrumbs = []
@@ -392,6 +400,7 @@
         inner_breadcrumbs.append({
             'dir_name': dir_name,
             'file_id': inv.path2id('/'.join(dir_parts[:index + 1])),
+            'suffix': '/' + view ,
         })
     return inner_breadcrumbs
 

